/************************************************/
/*                                              */
/*     common control                           */
/*                                              */
/************************************************/
#include <ctype.h>
#include "kernel.h"
#include "common.h"

const char clibCvT[] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
};

/*
 * soft delay
 */
void wait(long msec)
{
    long i;

    for(i=0L;i<msec*1000L;i++) {
        ;
    }
}

/*
 * hex to char
 */
char h2c(unsigned char hex)
{
    return((char)clibCvT[(unsigned char)(hex & 0x0Fu)]);
}

/*
 * hex to 1char ascii
 */
void h2b(char *str, unsigned char ch)
{
    *str++ = clibCvT[(unsigned char)((ch & 0xF0u)>>4)];
    *str++ = clibCvT[(unsigned char)(ch & 0x0Fu)];
    *str++ = 0;
}

/*
 * hex to ascii
 */
void h2a(char *str, unsigned long hex)
{
    unsigned long dat;
    char f,i;

    if(hex == 0uL) {
        *str++ = '0';
    } else {
        f = 0;
        for(i=0;i<8;i++) {
            dat = hex & 0xF0000000uL;
            if(dat != 0uL) {
                dat >>= 28;
                *str++ = clibCvT[dat];
                f++;
            } else {
                if(f != 0) {
                    *str++ = '0';
                }
            }
            hex <<= 4;
        }
    }
    *str = 0 ;
}

/*
 * hex to ascii(8char)
 */
void h2a8(char *str, unsigned long hex)
{
    unsigned long dat;
    char i;

    for(i=0;i<8;i++) {
        dat = hex >> 28;
        *str++ = clibCvT[dat];
        hex <<= 4;
    }
    *str = 0 ;
}

/*
 * hex to ascii(6char)
 */
void h2a6(char *str, unsigned long hex)
{
    unsigned long dat;
    char i;

    hex <<= 8;
    for(i=0;i<6;i++) {
        dat = hex >> 28;
        *str++ = clibCvT[dat];
        hex <<= 4;
    }
    *str = 0 ;
}
/*
 * hex to ascii(4char)
 */
void h2a4(char *str, unsigned long hex)
{
    unsigned long dat;
    char i;

    hex <<= 16;
    for(i=0;i<4;i++) {
        dat = hex >> 28;
        *str++ = clibCvT[dat];
        hex <<= 4;
    }
    *str = 0 ;
}

/*
 * integer to ASCII string
 */
void i2a(char *str, unsigned long dec)
{
    char  nums[10];
    long  n;

    n = 0L;
    while(dec > 0uL) {
        nums[n++] = (char)(dec % 10uL);
        dec = dec / 10uL;
    }

    if(n > 0L) {
        for(n--;n>=0L; n--) {
            *str++ = clibCvT[nums[n]];
        }
    } else {
        *str++ = '0';
    }
    *str = 0 ;
}

/*
 * ASCII string to integer
 */
char a2i(char *str, unsigned long *dec)
{
    char  ercd;
    unsigned long cnt;

    ercd = 0;
    cnt = 0uL;
    while((*str != '\0') && (ercd == 0)) {
        if((*str >= '0') && (*str <= '9')){
            cnt *= 10uL;
            cnt += ((unsigned long)*str & 0x0FuL);
            *str++;
        }else{
            ercd = -1;
        }
    }
    *dec = cnt;
    return(ercd);
}

/*
 * ASCII string to hex
 */
char a2h(char *str, unsigned long *hex)
{
    char  ercd;
    unsigned long cnt;

    ercd = 0;
    cnt = 0uL;
    while((*str != '\0') && (ercd == 0)){
        cnt *= 16uL;
        if((*str >= '0') && (*str <= '9')) {
            cnt += ((unsigned long)*str & 0x0FuL);
        } else if( ((*str >= 'a') && (*str <= 'f')) ||
                   ((*str >= 'A') && (*str <= 'F')) ){
            cnt += ((unsigned long)*str & 0x0FuL) + 9uL;
        }else{
            ercd = -1;
        }
        *str++;
    }
    *hex = cnt;
    return(ercd);
}


/*
 * ASCII string to hex(1byte)
 */
long a2h1(char ch)
{
    long cnt;

    cnt = 0L;
    if((ch >= '0') && (ch <= '9')) {
        cnt = (long)((unsigned char)ch & 0x0Fu);
    } else if( ((ch >= 'a') && (ch <= 'f')) ||
               ((ch >= 'A') && (ch <= 'F')) ){
        cnt = (long)((unsigned char)ch & 0x0Fu) + 9L;
    } else {
        ;
    }
    return(cnt);
}

/*
 * ASCII string to hex(1byte)
 */
void a2b(char *str, unsigned char *hex)
{
    unsigned long  data, i;

    for(i=0uL; i<2uL; i++) {
        data = data * 16uL;
        if((*str >= '0') && (*str <= '9')) {
          data += (unsigned long)(*str++ & 0x0fu);
        } else {
          data += (unsigned long)(*str++ & 0x0fu) + 9uL;
        }
    }
    *hex = data;
}

/*
 * Remove Space
 */
void rm_space(char *str)
{
    char *in,*out;
    char flg;

    in  = str;
    out = str;
    flg = 1;
    while(*in != 0) {
        if(*in == CODE_SP) {
            in++;
            while((*in == CODE_SP) && (*in != 0)) {
              in++;
            }
            if((flg == 0) && (*in != 0)) {
                *out++ = CODE_SP;
            }
        } else {
            *out++ = *in++;
            flg = 0;
        }
    }
    *out = 0;
}

/*
 * String to lower
 */
void str2low(char *str)
{
    for(;*str!=0;str++) {
        *str = (char)tolower(*str);
    }
}

/*
 * Get word
 */
char *get_word(char *str)
{
    while((*str != CODE_SP) && (*str != 0)) {
        str++;
    }
    if(*str == 0) {
        *str = 0;
    } else {
        *str++ = 0;
    }
    return(str);
}


/*
 * Get word
 */
char *get_word_token(char *str, char token)
{
    while((*str != token) && (*str != 0)) {
        str++;
    }
    if(*str == 0) {
        *str = 0;
    } else {
        *str++ = 0;
    }
    return(str);
}

