/************************************************/
/*                                              */
/*     SetupCamera                              */
/*                                              */
/************************************************/
#include <stdio.h>
#include <ctype.h>
#include "ipxdef.h"
#include "ipxsys.h"
#include "ipxprot.h"

#include "itron.h"
#include "kernel.h"

#include "common.h"

int cameraid;
int sci_bps, sci_lng, sci_par, sci_stp;
int mode;
int crmode;
int echomode;

const char char_ret[2] = { CODE_LF, CODE_CR };
const char char_lf[1]  = { CODE_LF };
const char char_cr[1]  = { CODE_CR };
const char char_bs[4]  = { CODE_BS, CODE_SP, CODE_BS, 0 };

const char ver[] = "SetupCamera Ver1.01\n";

/******************************************************************************
 [ ֐ ] checkhex
 [    ] void checkhex( void );
 [    ] 
 [ ߂l ] 
******************************************************************************/
int checkhex(char asc)
{
	int rc;
	
	rc = -1;
	if((asc >= '0') && (asc <= '9')) {
		rc = 0;
	} else if((asc >= 'A') && (asc <= 'F')) {
		rc = 0;
	} else if((asc >= 'a') && (asc <= 'f')) {
		rc = 0;
	}
	return rc;
}

/******************************************************************************
 [ ֐ ] get_stdin
 [    ] void get_stdin( void );
 [    ] 
 [ ߂l ] 
******************************************************************************/
char get_stdin(void)
{
	char ch;
	char str[12];
	int loop, i;
	long time;
	
	if(mode == 0) {
		/*----------------------------------------
			ASCII[h
		----------------------------------------*/
		ch = charget();
		if(echomode == 1) {
			if(ch == CODE_BS) { 
				printf("%s", char_bs);
			} else {
				printf("%c", ch&255);
			}
		}
	} else if(mode == 1) {
		/*----------------------------------------
			BIN[h
		----------------------------------------*/
		loop = 1;
		while(loop) {
			ch = charget();
			switch(ch) {
			  case CODE_ESP:	/* [ESC]->exit */
			  case CODE_TAB:	/* [TAB]->exit */
			  case CODE_CR:
			  case CODE_LF:
				loop = 0;
				break;
			  case CODE_SP:
				printf("\n");
				break;
			  case '~':			/* BINf[^ */
				if(echomode == 1) {
					printf("(~");
				}
				/* 2 */
				str[0] = charget();
				if(echomode == 1) {
					printf("%c", str[0]);
				}
				/* 3 */
				str[1] = charget();
				if(echomode == 1) {
					printf("%c)", str[1]);
				}
				if((checkhex(str[0])==0) && (checkhex(str[1])==0)) {
					a2b(&str[0], &ch);
					loop = 0;
				} else {
					printf("invalid data\n");
				}
				break;
			  case '#':			/* DelayR}h */
				/* 2 */
				str[0] = charget();
				/* 3 */
				str[1] = charget();
				if((checkhex(str[0])==0) && (checkhex(str[1])==0)) {
					a2b(&str[0], &ch);
					time=ch&255;
					tslp_tsk(time);
				} else {
					printf("invalid data\n");
				}
				break;
			  default:
				if(echomode == 1) {
					printf("%c", ch);
				}
				loop = 0;
				break;
			}
		}
	} else {
		/*----------------------------------------
			???[h(ESCI)
		----------------------------------------*/
		ch = CODE_ESP;
	}
	return ch;
}

/******************************************************************************
 [ ֐ ] scircv_tsk
 [    ] void scircv_tsk( void );
 [    ] 
 [ ߂l ] 
******************************************************************************/
void scircv_tsk(void)
{
	int ret;
	char ch;
	char str[12];

	while(1) {
		ret = scicom_Recv(2, &ch, 1);
		if(ret < 0) {
			printf( "scicom_Send ERROR %d\n", ret);
			return;
		}
		if(mode ==0) {
			/*----------------------------------------
				ASCII[h
			----------------------------------------*/
			printf("%c", ch);

			if(crmode==1) {
				if(ch == CODE_LF) {
					printf("%c", CODE_CR);
				}
				if(ch == CODE_CR) {
					printf("%c", CODE_LF);
				}
			}
		} else if(mode ==1) {
			/*----------------------------------------
				BIN[h
			----------------------------------------*/
			if(iscntrl(ch)) {
				printf("[~%02x]", ch&255);
			} else {
				printf("[%c]", ch&255);
			}
		} else {
			/*----------------------------------------
				???[h(Ȃ)
			----------------------------------------*/
			;
		}
	}
}

/******************************************************************************
 [ ֐ ] main
 [    ] void main( void );
 [    ] 
 [ ߂l ] 
******************************************************************************/
void main( int argc, int **argv )
{
	int boardNo = 0;
	int opt = 0;

	int ret,i,data,len,loop;
	char *ptr, *next;
	SCICOM_SETUP_INFO sciinfo;
	T_CTSK ctsk;
	ID tskid;

	char ch;

#ifdef NVPAX230
	unsigned short set_fpgadata;
	unsigned short save_fpgadata;
#endif
#ifdef SVPAX330
	unsigned char  set_fpgadata;
	unsigned char  save_fpgadata;
#endif

	cameraid = 0;
	sci_bps  = SCICOM_38400;
	sci_lng  = SCICOM_8Bit;
	sci_par  = SCICOM_NonParity;
	sci_stp  = SCICOM_StopBit1;
	mode     = 0;
    crmode   = 1;
	echomode = 0;

	/*----------------------------------------
		`FbN
	----------------------------------------*/
	/*
	 * SetupCamera /ch=[0`4](NVP-AX230)  [0`0](SVP-Ax330)
	 *             /mode=[ascii|binary]
	 *             /sci=[1200|2400|4800|9600|19200|38400|57600]-[8|7]-[N|E|P]-[1|2]
	 *             /crmode=[1|2|3]
	 *             /echo=[ON|OFF]
	 *             /ver
	 *             /machine
	 *
	 *  /ch      : J`lw(0`4)
	 *             (ftHg0)
	 *  /mode    : ascii(asc)  = AXL[[h ... 𑗎M
	 *             binary(bin) = oCi[h ... oCif[^𑗎M
	 *             (ftHgascii)
	 *  /sci     : VA[h
	 *              a-b-c-d
	 *              a = rbg[g(1200,2400,4800,9600,19200,38400,57600) 
	 *              b = f[^rbg(8,7)
	 *              c = peB(N,E,P)
	 *              d = Xgbvrbg(1,2)
	 *             (ftHg38400-8-N-1)
	 *  /crmode  : s[h
	 *              1 = CR->CR+LF, LF->CR+LF
	 *              2 = CR->CR, LF->LF
	 *              3 = CR->LF, LF->LF
	 *              3 = CR->CR, LF->CR
	 *             (ftHg1)
	 *  /etho    : ON  = GR[obNON
	 *             OFF = GR[obNOFF
	 *             (ftHgOFF)
	 *  /ver     : o[W
	 *  /machine : Ή{[h
	 *             
	 *
	 */
	for(i=1;i<argc;i++) {
		str2low(argv[i]);
		if(strncmp(argv[i], "/ch=", 4)==0) {
			/* CH`FbN */
			ptr  = (char *)argv[i];
			ptr += 4;
			if(a2i(ptr, (unsigned long *)&data)==0) {
				if((data >= CAMERACH_MIN) && (data <= CAMERACH_MAX)) {
					cameraid = data;
				} else {
					printf("invaild camera-id\n");
					return;
				}
			} else {
				printf("invaild camera-id\n");
				return;
			}
		} else if(strncmp(argv[i], "/mode=", 6)==0) {
			/* MODE`FbN */
			ptr  = (char *)argv[i];
			ptr += 6;
			if(strcmp(ptr, "ascii") == 0) {
				mode = 0;
			} else if(strcmp(ptr, "asc") == 0) {
				mode = 0;
			} else if(strcmp(ptr, "binary") == 0) {
				mode = 1;
			} else if(strcmp(ptr, "bin") == 0) {
				mode = 1;
			} else {
				printf("invaild mode\n");
				return;
			}
		} else if(strncmp(argv[i], "/sci=", 5)==0) {
			/* SCI`FbN */
			ptr  = (char *)argv[i];
			ptr += 5;

			next = get_word_token(ptr, '-');

			if(strcmp(ptr, "1200") == 0) {
				sci_bps = SCICOM_1200;
			} else if(strcmp(ptr, "2400") == 0) {
				sci_bps = SCICOM_2400;
			} else if(strcmp(ptr, "4800") == 0) {
				sci_bps = SCICOM_4800;
			} else if(strcmp(ptr, "9600") == 0) {
				sci_bps = SCICOM_9600;
			} else if(strcmp(ptr, "19200") == 0) {
				sci_bps = SCICOM_19200;
			} else if(strcmp(ptr, "38400") == 0) {
				sci_bps = SCICOM_38400;
			} else if(strcmp(ptr, "38400") == 0) {
				sci_bps = SCICOM_57600;
			} else {
				printf("invaild bit-rate\n");
				return;
			}

			ptr = next;
			next = get_word_token(ptr, '-');
			if(strcmp(ptr, "8") == 0) {
				sci_lng = SCICOM_8Bit;
			} else if(strcmp(ptr, "7") == 0) {
				sci_lng = SCICOM_7Bit;
			} else {
				printf("invaild bit count\n");
				return;
			}

			ptr = next;
			next = get_word_token(ptr, '-');
			if(strcmp(ptr, "n") == 0) {
				sci_par = SCICOM_NonParity;
			} else if(strcmp(ptr, "e") == 0) {
				sci_par = SCICOM_EvenParity;
			} else if(strcmp(ptr, "p") == 0) {
				sci_par = SCICOM_OddParity;
			} else {
				printf("invaild parity\n");
				return;
			}

			ptr = next;
			next = get_word_token(ptr, '-');
			if(strcmp(ptr, "1") == 0) {
				sci_stp = SCICOM_StopBit1;
			} else if(strcmp(ptr, "2") == 0) {
				sci_stp = SCICOM_StopBit2;
			} else {
				printf("invaild stop-bit\n");
				return;
			}

			ptr = next;
			if(ptr != 0) {
				if(*ptr != 0) {
					printf("invaild sci option\n");
					return;
				}
			}
		} else if(strncmp(argv[i], "/crmode=", 8)==0) {
			/* CRMODE`FbN */
			ptr  = (char *)argv[i];
			ptr += 8;
			if(strcmp(ptr, "1") == 0) {
				crmode = 1;
			} else if(strcmp(ptr, "2") == 0) {
				crmode = 2;
			} else if(strcmp(ptr, "3") == 0) {
				crmode = 3;
			} else if(strcmp(ptr, "4") == 0) {
				crmode = 4;
			} else {
				printf("invaild mode\n");
				return;
			}
		} else if(strncmp(argv[i], "/echo=", 6)==0) {
			/* ECHO`FbN */
			ptr  = (char *)argv[i];
			ptr += 6;
			if(strcmp(ptr, "on") == 0) {
				echomode = 1;
			} else if(strcmp(ptr, "off") == 0) {
				echomode = 0;
			} else {
				printf("invaild mode\n");
				return;
			}
		} else if(strncmp(argv[i], "/ver", 4)==0) {
			printf(ver);
			return;
		} else if(strncmp(argv[i], "/machine", 8)==0) {
#ifdef NVPAX230
			printf("NVP-Ax230\n");
#endif
#ifdef SVPAX330
			printf("SVP-Ax330\n");
#endif
			return;
		} else {
			/* Invalid optionG[ */
			printf("invaild option\n");
			return;
		}
	}

	/*----------------------------------------
		摜VXȅ
	----------------------------------------*/
	ret = StartIP( boardNo, opt );
	if( ret ){
		printf( "StartIP ERROR : %08X\n", ret );
		return;
	}

	/*----------------------------------------
		SCI
	----------------------------------------*/
	sciinfo.Rate      = sci_bps;
	sciinfo.Lng       = sci_lng;
	sciinfo.Par       = sci_par;
	sciinfo.Stop      = sci_stp;
	sciinfo.RcvBufSiz = 1024;
	sciinfo.SndBufSiz = 1024;

	ret = scicom_Setup(2, &sciinfo);
	if(ret) {
		printf( "scicom_Setup ERROR(%d)\n", ret);
	} else {
		/*----------------------------------------
			FPGAݒ(JVAݒ)
		----------------------------------------*/
#ifdef NVPAX230
		ReadMapDevice(0x18000016uL, 2, &save_fpgadata, 1);

		set_fpgadata  = save_fpgadata;
		set_fpgadata &= 0xFCu;
		set_fpgadata |= (unsigned short)cameraid;
		WriteMapDevice(0x18000016uL, 2, &set_fpgadata, 1);
#endif
#ifdef SVPAX330
		ReadMapDevice(0xBA000010uL, 1, &save_fpgadata, 1);

		set_fpgadata  = save_fpgadata;
		set_fpgadata &= 0xFCu;
		set_fpgadata |= 0x02u;
		WriteMapDevice(0xBA000010uL, 1, &set_fpgadata, 1);
#endif

		/*----------------------------------------
			M^XN
		----------------------------------------*/
		memset(&ctsk,0,sizeof(T_CTSK));
		ctsk.tskatr  = TA_HLNG | TA_ACT | TA_COP1 | TA_COP2;
		ctsk.exinf   = 0;
		ctsk.stksz   = 0x1000;					// X^bNTCY
		ctsk.stk     = NULL;
		ctsk.itskpri = TASK_PRI_NORMAL;			// vCIeB
		ctsk.task    = (FP)scircv_tsk;			// W[
		tskid = acre_tsk(&ctsk);
		if(tskid < 0) {
			printf( "acre_tsk ERROR\n" );
			return;
		}

		/*----------------------------------------
			STDIO-CAMERAʐM
		----------------------------------------*/
		loop = 1;
		while(loop) {
			ch = get_stdin();	/* 1oCg(bin[hAecho[h */
			if((ch == CODE_ESP) || (ch == CODE_TAB)) {
				loop=0;
				break;
			}
			if(crmode==1) {
				if((ch == CODE_CR) || (ch == CODE_LF)) {
					ret = scicom_Send(2, (char*)&char_ret[0], 2);
				} else {
					ret = scicom_Send(2, &ch, 1);
				}
			} else if(crmode==2) {
				ret = scicom_Send(2, &ch, 1);
			} else if(crmode==3) {
				if((ch == CODE_CR) || (ch == CODE_LF)) {
					ret = scicom_Send(2, (char*)&char_lf[0], 1);
				} else {
					ret = scicom_Send(2, &ch, 1);
				}
			} else if(crmode==4) {
				if((ch == CODE_CR) || (ch == CODE_LF)) {
					ret = scicom_Send(2, (char*)&char_cr[0], 1);
				} else {
					ret = scicom_Send(2, &ch, 1);
				}
			} else {
				;
			}
		}

		/*----------------------------------------
			M^XNI/폜
		----------------------------------------*/
		ter_tsk(tskid);
		del_tsk(tskid);

		/*----------------------------------------
			FPGAݒ(JVAݒ)
		----------------------------------------*/
#ifdef NVPAX230
		WriteMapDevice(0x18000016uL, 2, &save_fpgadata, 1);		
#endif
#ifdef SVPAX330
		WriteMapDevice(0xBA000010uL, 1, &save_fpgadata, 1);
#endif
	}
	printf("\n");

	/*----------------------------------------
		[gR}h̎gp~
	----------------------------------------*/
	StopIP( boardNo );
}
